/*
 * componentconfigurator.h
 * Copyright 2005 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors:
 *     Steve Grubb <sgrubb@redhat.com>
 */
#ifndef COMPONENTCONFIGURATOR_H
#define COMPONENTCONFIGURATOR_H
#include <string>
#include "componentrepository.h"

/**
 * Class ComponentConfigurator
 * This class follows the Singleton Pattern. This makes it accessible from
 * every class. Care must be taken in programming to ensure that proper
 * locking is observed in all methods.
 */
class ComponentConfigurator {
/**
 * Public stuff
 */
public:
	/**
	 * Fields
	 */
	/**
	 * 
	 */
	/**
	 * Constructors
	 */
	/**
	 * Accessor Methods
	 */
	/**
	 * Operations
	 */
	/**
	 * This method is part of the Singleton Pattern. It is used to
	 * create the class. It will return a reference to itself.
	 */
	static ComponentConfigurator *instance ()
	{
		if (!_instance) {
			static ComponentConfigurator theInstance;
			_instance = &theInstance;
		}
		return _instance;
	}
		
	
	/**
	 * This method will start the init process for the whole program. It
	 * should only be invoked once.
	 */
	int init ();
		
	
	/**
	 * This method tells the configurator to read the config file and
	 * reconfigure all components without taking down the application.
	 */
	void reconfigure ();
		
	
	/**
	 * This method is called to stop all processing before terminating
	 * the application or reconfiguring the plugins.
	 */
	void suspend_all ();
		
	
/**
 * Protected stuff
 */
protected:
	/**
	 * Fields
	 */
	/**
	 * 
	 */
	/**
	 * Constructors
	 */
	/**
	 * Accessor Methods
	 */
	/**
	 * Operations
	 */
/**
 * Private stuff
 */
private:
	/**
	 * Fields
	 */
	/**
	 * This is used to determine if the class needs to be created or just
	 * referenced. Its values are 0 and 1.
	 */
	static  ComponentConfigurator *_instance;
	/**
	 * This class owns the Component Repository.
	 */
	 ComponentRepository repo;
	/**
	 * This is the location where the config file resides.
	 */
	 const char *config_file;
	/**
	 * 
	 */
	/**
	 * Constructors
	 */
	/**
         * Empty Constructor
         */
        ComponentConfigurator( ) { };
        /**
         * Empty Constructor
         */
        ComponentConfigurator(const ComponentConfigurator&);
        /**
         * Empty Constructor
         */
        ComponentConfigurator& operator=(const ComponentConfigurator&);

        /**
         * Empty Constructor
         */
        virtual ~ComponentConfigurator() { _instance = 0; }

	/**
	 * Accessor Methods
	 */
		
	
	/**
	 * Get the value of repo
	 * This class owns the Component Repository.
	 * @return the value of repo
	 */
	ComponentRepository get_repo ( );
		
	
	/**
	 * Set the value of repo
	 * This class owns the Component Repository.
	 * @param value the value of repo
	 */
	void set_repo (ComponentRepository value );
		
	
	/**
	 * Get the value of config_file
	 * This is the location where the config file resides.
	 * @return the value of config_file
	 */
	const char *get_config_file ( );
		
	
	/**
	 * Set the value of config_file
	 * This is the location where the config file resides.
	 * @param value the value of config_file
	 */
	void set_config_file (const char *value );
		
	
	/**
	 * Operations
	 */
	/**
	 * This method causes the class to read the config file located by
	 * the config_file attribute.
	 */
	int read_config ();
		
	/**
	 * Read a line of the config file
	 * @param f the FILE handle
	 * @param buf the buffer to put the line in
	 */
	char *get_line( FILE *f, char *buf );	
};
#endif //COMPONENTCONFIGURATOR_H

